import { __awaiter, __values, __generator, __extends } from 'tslib';
import { auth, firestore, storage } from 'firebase';
import { Injectable, NgZone, NgModule } from '@angular/core';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { CommonModule } from '@angular/common';

var Library = /** @class */ (function () {
    function Library() {
    }
    Library.httpBuildQuery = function (params) {
        if (Library.isEmpty(params)) {
            return null;
        }
        var keys = Object.keys(params);
        if (keys.length === 0) {
            return null;
        }
        var esc = encodeURIComponent;
        var query = keys
            .map(function (k) { return esc(k) + '=' + esc(params[k]); })
            .join('&');
        return query;
    };
    Library.segment = function (str, separator, n) {
        if (separator === void 0) { separator = ' '; }
        if (n === void 0) { n = 0; }
        if (typeof str !== 'string') {
            return null;
        }
        if (typeof separator !== 'string' || !separator) {
            return null;
        }
        if (str) {
            var re = str.split(separator);
            if (re[n] !== void 0 && re[n]) {
                return re[n];
            }
        }
        return null;
    };
    Library.isEmpty = function (what) {
        if (!what) {
            return true;
        }
        if (typeof what === 'object') {
            return Object.keys(what).length === 0;
        }
        if (Array.isArray(what)) {
            return what.length === 0;
        }
        return false;
    };
    Library.isEqual = function (a, b) {
        if (typeof a === 'object' && typeof b === 'object') {
            var aKeys = Object.keys(a);
            var bKeys_1 = Object.keys(b);
            if (aKeys.length !== bKeys_1.length) {
                return false;
            }
            return aKeys.findIndex(function (v, i) { return v !== bKeys_1[i]; }) === -1;
        }
        else if (Array.isArray(a) && Array.isArray(b)) {
            if (a.length !== b.length) {
                return false;
            }
            else {
            }
        }
        else {
            return a === b;
        }
    };
    Library.isString = function (str) {
        return typeof str === 'string';
    };
    Library.sanitize = function (obj) {
        if (obj) {
            if (typeof obj === 'object') {
                Object.keys(obj).forEach(function (key) { return obj[key] === undefined && delete obj[key]; });
            }
        }
        if (obj && obj['password'] !== void 0) {
            delete obj['password'];
        }
        return obj;
    };
    Library.removeSpaceBetween = function (separator, str) {
        if (!str) {
            return str;
        }
        else {
            return str.split(separator).map(function (s) { return s.trim(); }).join(separator);
        }
    };
    Library.getBrowserLanguage = function (full) {
        if (full === void 0) { full = false; }
        var nav = window.navigator;
        var browserLanguagePropertyKeys = ['language', 'browserLanguage', 'systemLanguage', 'userLanguage'];
        var ln = null;
        if (Array.isArray(nav.languages)) {
            for (var i = 0; i < nav.languages.length; i++) {
                var language = nav.languages[i];
                if (language && language.length) {
                    ln = language;
                    break;
                }
            }
        }
        for (var i = 0; i < browserLanguagePropertyKeys.length; i++) {
            var language = nav[browserLanguagePropertyKeys[i]];
            if (language && language.length) {
                ln = language;
                break;
            }
        }
        if (ln) {
            if (full === false) {
                ln = ln.substring(0, 2);
            }
        }
        return ln;
    };
    Library.patchMarker = function (str, info) {
        if (info === void 0) { info = null; }
        if (info === null || typeof info !== 'object') {
            return str;
        }
        var keys = Object.keys(info);
        if (!keys.length) {
            return str;
        }
        try {
            for (var keys_1 = __values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                var k = keys_1_1.value;
                str = str.replace('#' + k, ((info[k])));
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (keys_1_1 && !keys_1_1.done && (_a = keys_1.return)) _a.call(keys_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return str;
        var e_1, _a;
    };
    return Library;
}());
var COLLECTIONS = {
    USERS: 'users',
    SETTINGS: 'settings',
    CATEGORIES: 'categories',
    POSTS: 'posts',
    LIKES: 'likes',
    DISLIKES: 'dislikes',
    COMMENTS: 'comments'
};
var POST_DELETED = 'This post has been deleted!';
var UNKNOWN = 'unknown-error';
var FIREBASE_API_ERROR = 'firebase-api-error';
var NOT_FOUND = 'not-found';
var ALREADY_EXISTS = 'already-exists';
var PERMISSION_DENIED = 'permission-denied';
var RESOURCE_EXHAUSTED = 'resource-exhausted';
var USER_IS_NOT_LOGGED_IN = 'You are not logged in';
var NO_DOCUMENT_ID = 'no-document-id';
var DOCUMENT_ID_TOO_LONG = 'document-id-too-long';
var DOCUMENT_ID_CANNOT_CONTAIN_SLASH = 'document-id-cannot-cotain-slash';
var COLLECTION_NOT_EMPTY = 'collection-not-empty';
var CATEGORY_ID_EMPTY = NO_DOCUMENT_ID;
var CATEGORY_EXISTS = ALREADY_EXISTS;
var CATEGORY_DOES_NOT_EXIST = NOT_FOUND;
var POST_ID_NOT_EMPTY = 'post-id-not-empty';
var POST_ID_EMPTY = 'post-id-empty';
var ALREADY_LIKED = 'already-liked-or-disliked';
var EMAIL_ALREADY_IN_USE = 'auth/email-already-in-use';
var USER_NOT_FOUND = 'auth/user-not-found';
var INVALID_EMAIL = 'auth/invalid-email';
var WEAK_PASSWORD = 'auth/weak-password';
var EXPIRED_ID_TOKEN = 'auth/id-token-revoked';
var WRONG_PASSWORD = 'auth/wrong-password';
var PASSWORD_TOO_LONG = 'password-too-long';
var en = {
    'HOME': 'Home'
};
en[UNKNOWN] = 'Unkown error. #info';
en[FIREBASE_API_ERROR] = 'Firebase API error. #info';
en[CATEGORY_ID_EMPTY] = 'Category ID is empty.';
en[NOT_FOUND] = 'Document was not found. Document ID: #documentID';
en[ALREADY_EXISTS] = 'Category is already exists. Category ID: #categoryID';
en[DOCUMENT_ID_TOO_LONG] = 'Document ID is too long. Document ID: #documentID';
en[PERMISSION_DENIED] = 'Permission denied or insufficient permission. #info';
en[POST_ID_EMPTY] = 'Post ID is empty.';
en[POST_ID_NOT_EMPTY] = 'Post ID is not empty.';
en[EMAIL_ALREADY_IN_USE] = 'Email address already in use. #info';
en[USER_NOT_FOUND] = 'User was not found in collection. #info';
en[INVALID_EMAIL] = 'Invalid email, #info';
en[WEAK_PASSWORD] = 'Weak password, #info';
en[PASSWORD_TOO_LONG] = 'Password length exceeds at maximum 128 characters.';
en[EXPIRED_ID_TOKEN] = 'ID token is revoked/expired. Login again to get a new id token.';
en[WRONG_PASSWORD] = 'Password was incorrect, #info';
en[USER_IS_NOT_LOGGED_IN] = 'You are not logged in.';
var domain;
domain = 'localhost';
var COLLECTION_ROOT = 'fire-library';
var COLLECTION_DOMAIN = domain;
var Base = /** @class */ (function () {
    function Base(collectionName) {
        if (collectionName === void 0) { collectionName = ''; }
        this.collectionName = collectionName;
        this.auth = null;
        this.db = null;
        this.storage = null;
        this.auth = Base.firebase.auth();
        this.db = Base.firebase.firestore();
        this.storage = Base.firebase.storage();
    }
    Base.configure = function (config) {
        Base.firebase = config.firebaseApp;
        Base.functions = config.functions;
    };
    Base.prototype.version = function () {
        return '0.0.2';
    };
    Object.defineProperty(Base.prototype, "settings", {
        get: function () {
            return Base.settings;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Base.prototype, "collectionRoot", {
        get: function () {
            return Base.collectionRoot;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Base.prototype, "collectionDomain", {
        get: function () {
            return Base.collectionDomain;
        },
        enumerable: true,
        configurable: true
    });
    Base.prototype.setSettings = function (obj) {
        if (obj) {
            Base.settings = Object.assign(Base.settings, obj);
        }
    };
    Object.defineProperty(Base.prototype, "http", {
        get: function () {
            return Base.http;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Base.prototype, "collection", {
        get: function () {
            if (this.collectionName) {
                return this.db.collection(this.collectionRoot)
                    .doc(this.collectionDomain)
                    .collection(this.collectionName);
            }
            else {
                return null;
            }
        },
        enumerable: true,
        configurable: true
    });
    Base.prototype.collectionRef = function (name) {
        if (name) {
            return this.db.collection(this.collectionRoot)
                .doc(this.collectionDomain)
                .collection(name);
        }
        else {
            return null;
        }
    };
    Object.defineProperty(Base.prototype, "settingsReference", {
        get: function () {
            return this.collectionRef(COLLECTIONS.SETTINGS);
        },
        enumerable: true,
        configurable: true
    });
    Base.prototype.success = function (data) {
        return {
            code: null,
            data: data
        };
    };
    Base.prototype.failure = function (obj, info) {
        if (info === void 0) { info = {}; }
        var e;
        if (typeof obj === 'string') {
            e = new Error(obj);
        }
        else {
            e = obj;
        }
        if (e['_count']) {
            e['_count']++;
            return Promise.reject(e);
        }
        if (this.isFirebaseError(e, info)) {
            this.translateFirebaseError(e, info);
        }
        else {
            e['code'] = e.message;
            e['message'] = this.translate(e.message, info);
        }
        if (e['code'] === e['message']) {
            e['message'] = "Error code - " + e['code'] + " - is not translated. Please translate it. It may be firebase error.";
        }
        e['_count'] = 1;
        return Promise.reject(e);
    };
    Base.prototype.isFirebaseError = function (e, info) {
        switch (e.code) {
            case 'not-found':
                var str = e.message;
                info['documentID'] = str.split('/').pop();
                return true;
            case INVALID_EMAIL:
            case WRONG_PASSWORD:
            case PERMISSION_DENIED:
            case ALREADY_EXISTS:
            case EMAIL_ALREADY_IN_USE:
            case WEAK_PASSWORD:
            case USER_NOT_FOUND:
            case EXPIRED_ID_TOKEN:
            case PASSWORD_TOO_LONG:
            case USER_IS_NOT_LOGGED_IN:
                info['info'] = e.message;
                return true;
            default:
                return false;
        }
    };
    Base.prototype.translateFirebaseError = function (e, info) {
        e['message'] = this.translate(e.code, info);
    };
    Base.prototype.translate = function (code, info) {
        return Library.patchMarker(this.getText(code), info);
    };
    Base.prototype.getText = function (code) {
        var ln = this.getLanguage();
        var text = null;
        if (this.getLanguage() !== 'en') {
            if (Base.texts[ln] !== void 0 && Base.texts[ln][code] !== void 0) {
                text = Base.texts[ln][code];
            }
        }
        if (!text) {
            if (Base.texts['en'][code] !== void 0) {
                text = Base.texts['en'][code];
            }
        }
        if (!text) {
            text = code;
        }
        return text;
    };
    Base.prototype.getLanguage = function () {
        return Base.language;
    };
    Base.prototype.setLanguage = function (ln) {
        Base.language = ln;
        if (ln === 'en') {
            return Promise.resolve();
        }
        if (Base.texts[ln]) {
            return Promise.resolve();
        }
        return this.http.get("/" + Base.languageFolder + "/" + ln + ".json").toPromise()
            .then(function (re) { return Base.texts[ln] = re; });
    };
    Base.prototype.checkDocumentIDFormat = function (documentID) {
        if (Library.isEmpty(documentID)) {
            return NO_DOCUMENT_ID;
        }
        if (documentID.length > 128) {
            return DOCUMENT_ID_TOO_LONG;
        }
        if (documentID.indexOf('/') !== -1) {
            return DOCUMENT_ID_CANNOT_CONTAIN_SLASH;
        }
        return null;
    };
    Base.prototype.getValidator = function (id) {
        return __awaiter(this, void 0, void 0, function () {
            var idCheck;
            return __generator(this, function (_a) {
                idCheck = this.checkDocumentIDFormat(id);
                if (idCheck) {
                    return [2 /*return*/, this.failure(new Error(idCheck), { documentID: id })];
                }
                return [2 /*return*/, null];
            });
        });
    };
    Base.prototype.get = function (id) {
        var _this = this;
        return this.getValidator(id)
            .then(function () {
            return _this.collection.doc(id).get();
        })
            .then(function (doc) {
            if (doc.exists) {
                return _this.success(doc.data());
            }
            else {
                return _this.failure(new Error(NOT_FOUND));
            }
        })
            .catch(function (e) { return _this.failure(e); });
    };
    Base.prototype.likeDocument = function (collectionRef) {
        var uid = auth().currentUser.uid;
        var ref = collectionRef.doc(uid);
        console.log("likeDocument path: ", ref.path);
        return ref;
    };
    Base.prototype.doLikeValidator = function (collectionRef) {
        var _this = this;
        console.log("doLikeValidator() on " + collectionRef.path);
        return this.likeDocument(collectionRef).get()
            .then(function (doc) {
            if (doc.exists) {
                console.log('likeValidator. already liked');
                return _this.failure(ALREADY_LIKED);
            }
            else {
                return null;
            }
        })
            .catch(function (e) {
            console.log("Caught on validation:Failed to get like/dislike document.This may be a permission error on security rule.");
            return _this.failure(e);
        });
    };
    Base.prototype.doLike = function (collectionRef) {
        var _this = this;
        console.log("doLike() on " + collectionRef.path);
        return this.doLikeValidator(collectionRef)
            .then(function () {
            console.log("validator passed. Going to like/dislike on " + collectionRef.path);
            return _this.likeDocument(collectionRef)
                .set({ time: firestore.FieldValue.serverTimestamp() });
        })
            .then(function () {
            console.log("like/dislike under " + collectionRef.path + " has been added: ");
            return _this.countLikes(collectionRef);
        })
            .catch(function (e) {
            if (e.code === ALREADY_LIKED) {
                console.log("already like/dislike it. Going to unlike/undislike");
                return _this.doUnlike(collectionRef);
            }
            else {
                console.log("doLike() failed because: ", e);
                return _this.failure(e);
            }
        });
    };
    Base.prototype.doUnlike = function (collectionRef) {
        var _this = this;
        console.log("Going to unlike/undislike on " + collectionRef.path);
        return this.likeDocument(collectionRef).delete()
            .then(function () {
            console.log("like/dislike has been deleted(unliked/undisliked) on " + collectionRef.path + " ");
            return _this.countLikes(collectionRef);
        })
            .catch(function (e) { return _this.failure(e); });
    };
    Base.prototype.countLikes = function (collectionRef) {
        console.log("countLikes() on " + collectionRef.path);
        return collectionRef.get()
            .then(function (snapshot) {
            var count = 0;
            if (snapshot.size > 2) {
                count = snapshot.size - 1;
            }
            else {
                snapshot.forEach(function (doc) {
                    if (doc && doc.exists) {
                        if (doc.id !== 'count') {
                            count++;
                        }
                    }
                });
            }
            console.log(collectionRef.path + " count: ", count);
            return collectionRef.doc('count').set({ count: count });
        })
            .then(function () {
            console.log(collectionRef.path + " counted: ");
        });
    };
    Base.prototype.checkInstall = function () {
        var _this = this;
        return this.settingsReference.doc('installed').get().then(function (doc) {
            if (doc && doc.exists) {
                return true;
            }
            else {
                return false;
            }
        })
            .then(function (re) { return _this.success({ installed: re }); })
            .catch(function (e) { return _this.failure(e); });
    };
    Base.prototype.install = function (options) {
        var _this = this;
        var ref = this.settingsReference.doc('admin');
        console.log("Going to set admin email on " + ref.path);
        return ref.set({ email: options.email }).then(function () {
            var installRef = _this.settingsReference.doc('installed');
            console.log("Admin email is set. Going to set installed.time at " + installRef.path);
            return installRef
                .set({ time: firestore.FieldValue.serverTimestamp() });
        })
            .then(function (re) { return _this.success(true); })
            .catch(function (e) { return _this.failure(e); });
    };
    return Base;
}());
Base.collectionRoot = COLLECTION_ROOT;
Base.collectionDomain = COLLECTION_DOMAIN;
Base.firebase = null;
Base.functions = false;
Base.http = null;
Base.language = 'en';
Base.languageFolder = 'assets/lang';
Base.texts = { en: en };
Base.settings = {};
var User = /** @class */ (function (_super) {
    __extends(User, _super);
    function User() {
        return _super.call(this, COLLECTIONS.USERS) || this;
    }
    Object.defineProperty(User.prototype, "isLogin", {
        get: function () {
            if (this.auth.currentUser) {
                return true;
            }
            else {
                return false;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(User.prototype, "isLogout", {
        get: function () {
            return !this.isLogin;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(User.prototype, "uid", {
        get: function () {
            if (this.auth.currentUser) {
                return this.auth.currentUser.uid;
            }
            else {
                return null;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(User.prototype, "email", {
        get: function () {
            if (this.auth.currentUser) {
                return this.auth.currentUser.email;
            }
            else {
                return null;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(User.prototype, "displayName", {
        get: function () {
            if (this.auth.currentUser) {
                return this.auth.currentUser.displayName;
            }
            else {
                return null;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(User.prototype, "photoURL", {
        get: function () {
            if (this.auth.currentUser) {
                return this.auth.currentUser.photoURL;
            }
            else {
                return null;
            }
        },
        enumerable: true,
        configurable: true
    });
    User.prototype.data = function () {
        var _this = this;
        var user = {
            email: this.email,
            displayName: this.displayName,
            photoURL: this.photoURL,
        };
        return this.collection.doc(this.uid).get().then(function (doc) {
            if (doc && doc.exists) {
                return _this.success({ user: Object.assign(user, doc.data()) });
            }
            else {
                return _this.failure(USER_NOT_FOUND);
            }
        })
            .catch(function (e) { return _this.failure(e); });
    };
    User.prototype.registerValidator = function (user) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (!Library.isString(user.password) || !Library.isString(user.email)) {
                    return [2 /*return*/, this.failure(FIREBASE_API_ERROR, { info: 'Password and Email should contain valid string.' })];
                }
                if (user.password.length > 128) {
                    return [2 /*return*/, Promise.reject(PASSWORD_TOO_LONG)];
                }
                return [2 /*return*/, null];
            });
        });
    };
    User.prototype.register = function (data) {
        var _this = this;
        return this.registerValidator(data)
            .then(function () {
            return _this.auth.createUserWithEmailAndPassword(data.email, data.password);
        })
            .then(function (user) {
            return _this.updateAuthentication(user, data);
        })
            .then(function (user) {
            console.log("Going to set user data under users collection: ");
            return _this.set(user, data);
        })
            .then(function (a) { return _this.success(a); })
            .catch(function (e) { return _this.failure(e); });
    };
    User.prototype.loginValidator = function (email, password) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (!Library.isString(email) || !Library.isString(password)) {
                    return [2 /*return*/, this.failure(FIREBASE_API_ERROR, { info: 'Both email and password should contain valid string.' })];
                }
                return [2 /*return*/, null];
            });
        });
    };
    User.prototype.login = function (email, password) {
        var _this = this;
        return this.loginValidator(email, password)
            .then(function () {
            return _this.auth.signInWithEmailAndPassword(email, password);
        })
            .then(function (a) { return _this.success(a); })
            .catch(function (e) { return _this.failure(e); });
    };
    User.prototype.logout = function () {
        this.auth.signOut();
    };
    User.prototype.updateAuthentication = function (user, data) {
        var up = {
            displayName: data.displayName,
            photoURL: data.photoURL
        };
        console.log('updateAuthentication: up: ', up);
        return user.updateProfile(Library.sanitize(up)).then(function (x) { return user; });
    };
    User.prototype.set = function (user, data) {
        var _this = this;
        delete data.displayName;
        delete data.photoURL;
        delete data.password;
        data.created = firestore.FieldValue.serverTimestamp();
        data.uid = this.uid;
        return this.create(data)
            .catch(function (e) {
            console.log("Failed to set data under users collection: ");
            return _this.failure(e);
        });
    };
    User.prototype.create = function (data) {
        var _this = this;
        var ref = this.collection.doc(data.uid);
        console.log("set at: " + ref.path + " with: ", data);
        return ref.set(data)
            .then(function () { return _this.success({ id: data.uid }); })
            .catch(function (e) { return _this.failure(e); });
    };
    User.prototype.update = function (data) {
        var _this = this;
        data.uid = this.uid;
        console.log('update: ', data);
        return this.collection.doc(data.uid).set(data)
            .then(function () { return _this.success({ id: data.uid }); })
            .catch(function (e) { return _this.failure(e); });
    };
    User.prototype.delete = function () {
        var _this = this;
        console.log('delete: ', this.uid);
        return this.collection.doc(this.uid).delete()
            .then(function () { return _this.success(null); })
            .catch(function (e) { return _this.failure(e); });
    };
    return User;
}(Base));
var Category = /** @class */ (function (_super) {
    __extends(Category, _super);
    function Category() {
        return _super.call(this, COLLECTIONS.CATEGORIES) || this;
    }
    Category.prototype.createValidator = function (category) {
        var _this = this;
        var idCheck = this.checkDocumentIDFormat(category.id);
        if (idCheck) {
            return this.failure(new Error(idCheck), { documentID: category.id });
        }
        return this.collection.doc(category.id).get()
            .then(function (doc) {
            if (doc.exists) {
                return _this.failure(new Error(CATEGORY_EXISTS));
            }
            else {
                return null;
            }
        })
            .catch(function (e) { return _this.failure(e); });
    };
    Category.prototype.create = function (category) {
        var _this = this;
        return this.createValidator(category)
            .then(function () {
            category.subcategories = Library.removeSpaceBetween(',', category.subcategories);
            category.created = firestore.FieldValue.serverTimestamp();
            var ref = _this.collection.doc(category.id);
            console.log("set at: " + ref.path);
            return ref.set(Library.sanitize(category));
        })
            .then(function () { return _this.success({ id: category.id }); })
            .catch(function (e) {
            return _this.failure(e);
        });
    };
    Category.prototype.editValidator = function (category) {
        return __awaiter(this, void 0, void 0, function () {
            var idCheck;
            return __generator(this, function (_a) {
                idCheck = this.checkDocumentIDFormat(category.id);
                if (idCheck) {
                    return [2 /*return*/, this.failure(new Error(idCheck), { documentID: category.id })];
                }
                return [2 /*return*/, null];
            });
        });
    };
    Category.prototype.edit = function (category) {
        var _this = this;
        return this.editValidator(category)
            .then(function () { return _this.collection.doc(category.id).update(Library.sanitize(category)); })
            .then(function () { return _this.success({ id: category.id }); })
            .catch(function (e) { return _this.failure(e); });
    };
    Category.prototype.deleteValidator = function (id) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            var idCheck;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        idCheck = this.checkDocumentIDFormat(id);
                        if (idCheck) {
                            return [2 /*return*/, this.failure(new Error(idCheck), { documentID: id })];
                        }
                        return [4 /*yield*/, this.collection.doc(id).get()
                                .then(function (doc) {
                                if (doc.exists) {
                                    var data = (doc.data());
                                    if (data.numberOfPosts) {
                                        return _this.failure(COLLECTION_NOT_EMPTY);
                                    }
                                    else {
                                        return null;
                                    }
                                }
                                else {
                                    return _this.failure(new Error(NOT_FOUND));
                                }
                            })];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    Category.prototype.delete = function (id) {
        var _this = this;
        return this.deleteValidator(id)
            .then(function () {
            return _this.collection.doc(id).delete();
        })
            .catch(function (e) { return _this.failure(e); });
    };
    Category.prototype.categories = function () {
        return this.collection.get().then(function (querySnapshot) {
            var categories = ([]);
            querySnapshot.forEach(function (doc) {
                categories.push((doc.data()));
            });
            return categories;
        });
    };
    Category.prototype.get = function (id) {
        return _super.prototype.get.call(this, id);
    };
    return Category;
}(Base));
var Post = /** @class */ (function (_super) {
    __extends(Post, _super);
    function Post() {
        var _this = _super.call(this, COLLECTIONS.POSTS) || this;
        _this.cursor = null;
        _this.categoryId = null;
        _this.pagePosts = {};
        _this.pagePostIds = [];
        _this._unsubscribeLikes = [];
        _this._unsubscribePosts = [];
        _this.unsubscribePage = null;
        _this.user = new User();
        return _this;
    }
    Post.prototype.createValidator = function (post) {
        if (this.user.isLogout) {
            return Promise.reject(new Error(USER_IS_NOT_LOGGED_IN));
        }
        if (post.id) {
            return Promise.reject(new Error(POST_ID_NOT_EMPTY));
        }
        if (Library.isEmpty(post.category)) {
            return Promise.reject(new Error(CATEGORY_ID_EMPTY));
        }
        return Promise.resolve(null);
    };
    Post.prototype.createSanitizer = function (post) {
        Library.sanitize(post);
        post.uid = this.user.uid;
        post.date = this.user.displayName;
        post.created = firestore.FieldValue.serverTimestamp();
        return post;
    };
    Post.prototype.create = function (post) {
        var _this = this;
        return this.createValidator(post)
            .then(function () {
            return _this.collection.add(_this.createSanitizer(post));
        })
            .then(function (doc) {
            return _this.success({ id: doc.id, post: post });
        })
            .catch(function (e) { return _this.failure(e); });
    };
    Post.prototype.editValidator = function (post) {
        if (this.user.isLogout) {
            return Promise.reject(new Error(USER_IS_NOT_LOGGED_IN));
        }
        if (Library.isEmpty(post.id)) {
            return Promise.reject(new Error(POST_ID_EMPTY));
        }
        return Promise.resolve(null);
    };
    Post.prototype.edit = function (post) {
        var _this = this;
        return (this.editValidator(post)
            .then(function () {
            Library.sanitize(post);
            post.updated = firestore.FieldValue.serverTimestamp();
            var ref = _this.collection.doc(post.id);
            console.log('update at: ', ref.path);
            return ref.update(post);
        })
            .then(function () {
            return _this.success({ id: post.id, post: post });
        })
            .catch(function (e) { return _this.failure(e); }));
    };
    Post.prototype.delete = function (id) {
        var post = {
            id: id,
            title: POST_DELETED,
            content: POST_DELETED,
            deleted: true
        };
        return this.edit(post);
    };
    Post.prototype.categoryChanged = function (category) {
        return this.categoryId !== category;
    };
    Post.prototype.resetCursor = function (category) {
        this.categoryId = category;
        this.cursor = null;
    };
    Post.prototype.resetLoadPage = function (category) {
        var reset = false;
        if (category) {
            if (category === 'all' || this.categoryId !== category) {
                this.pagePosts = {};
                this.pagePostIds = [];
                this.categoryId = category;
                this.resetCursor(category);
                this.unsubscribePosts();
                this.unsubscribeLikes();
                reset = true;
            }
        }
        return reset;
    };
    Post.prototype.unsubscribePosts = function () {
        if (this._unsubscribePosts.length) {
            this._unsubscribePosts.map(function (unsubscribe) { return unsubscribe(); });
        }
    };
    Post.prototype.unsubscribeLikes = function () {
        if (this._unsubscribeLikes.length) {
            this._unsubscribeLikes.map(function (unsubscribe) {
                unsubscribe();
            });
        }
    };
    Post.prototype.subscribePostChange = function (post) {
        if (!this.settings.listenOnPostChange) {
            return;
        }
        var path = this.post(post.id).path;
        var unsubscribe = this.post(post.id).onSnapshot(function (doc) {
            post = Object.assign(post, doc.data());
        });
        this._unsubscribePosts.push(unsubscribe);
    };
    Post.prototype.subscribeLikes = function (post) {
        if (!this.settings.listenOnPostLikes) {
            return;
        }
        var likeRef = this.likeColllection(post.id, COLLECTIONS.LIKES).doc('count');
        var subscribeLik = likeRef.onSnapshot(function (doc) {
            if (doc.exists) {
                var data = doc.data();
                post.numberOfLikes = data["count"];
            }
        });
        this._unsubscribeLikes.push(subscribeLik);
        var dislikeRef = this.likeColllection(post.id, COLLECTIONS.DISLIKES).doc('count');
        var subscribeDislike = dislikeRef.onSnapshot(function (doc) {
            if (doc.exists) {
                var data = doc.data();
                post.numberOfDislikes = data["count"];
            }
        });
        this._unsubscribeLikes.push(subscribeDislike);
    };
    Post.prototype.page = function (options) {
        var _this = this;
        var reset = this.resetLoadPage(options.category);
        var query = (this.collection);
        if (this.categoryId && this.categoryId !== 'all') {
            query = query.where('category', '==', this.categoryId);
        }
        query = query.orderBy('created', 'desc');
        if (this.cursor) {
            query = query.startAfter(this.cursor);
        }
        query = query.limit(options.limit);
        return (query.get().then(function (querySnapshot) {
            if (querySnapshot.docs.length) {
                querySnapshot.forEach(function (doc) {
                    var post = (doc.data());
                    post.id = doc.id;
                    post['date'] = (new Date(post.created)).toLocaleString();
                    _this.pagePosts[post.id] = post;
                    _this.pagePostIds.push(post.id);
                    _this.subscribePostChange(post);
                    _this.subscribeLikes(post);
                });
                _this.cursor = querySnapshot.docs[querySnapshot.docs.length - 1];
                if (reset) {
                    _this.subscribePostAdd(query);
                }
                return _this.pagePosts;
            }
            else {
                return [];
            }
        }));
    };
    Post.prototype.subscribePostAdd = function (query) {
        var _this = this;
        if (!this.settings.listenOnPostChange) {
            return;
        }
        if (this.unsubscribePage) {
            this.unsubscribePage();
        }
        this.unsubscribePage = query.limit(1).onSnapshot(function (snapshot) {
            snapshot.docChanges.forEach(function (change) {
                var doc = change.doc;
                if (doc.metadata.hasPendingWrites) {
                    console.log('pending', doc.metadata.hasPendingWrites, 'from cache: ', doc.metadata.fromCache);
                }
                else {
                    console.log('pending', doc.metadata.hasPendingWrites, 'type: ', change.type, 'from cache: ', doc.metadata.fromCache, doc.data());
                    var post = doc.data();
                    post.id = doc.id;
                    console.log("exists: " + _this.pagePosts[post.id]);
                    if (change.type === 'added' && _this.pagePosts[post.id] === void 0) {
                        _this.addPostOnTop(post);
                    }
                    else if (change.type === 'modified') {
                        _this.updatePost(post);
                    }
                    else if (change.type === 'removed') {
                        _this.removePost(post);
                    }
                }
            });
        });
    };
    Post.prototype.addPostOnTop = function (post) {
        if (this.pagePosts[post.id] === void 0) {
            console.log("addPostOnTop: ", post);
            this.pagePosts[post.id] = post;
            this.pagePostIds.unshift(post.id);
            this.subscribePostChange(post);
            this.subscribeLikes(post);
        }
    };
    Post.prototype.updatePost = function (post) {
        console.log('updatePost id: ', post.id);
        if (this.pagePosts[post.id]) {
            console.log("updatePost", post);
            this.pagePosts[post.id] = Object.assign(this.pagePosts[post.id], post);
        }
        else {
            this.addPostOnTop(post);
        }
    };
    Post.prototype.removePost = function (post) {
    };
    Post.prototype.stopLoadPage = function () {
        this.resetLoadPage(undefined);
        this.unsubscribeLikes();
        this.unsubscribePosts();
    };
    Post.prototype.post = function (postId) {
        return this.collection.doc(postId);
    };
    Post.prototype.likeColllection = function (postId, collectionName) {
        return this.collection.doc(postId)
            .collection(collectionName);
    };
    Post.prototype.like = function (postId) {
        return this.doLike(this.likeColllection(postId, COLLECTIONS.LIKES));
    };
    Post.prototype.dislike = function (postId) {
        return this.doLike(this.likeColllection(postId, COLLECTIONS.DISLIKES));
    };
    return Post;
}(Base));
var Data = /** @class */ (function (_super) {
    __extends(Data, _super);
    function Data() {
        var _this = _super.call(this, '') || this;
        _this.user = new User();
        return _this;
    }
    Data.prototype.getDataRef = function (file) {
        return storage().ref().child("" + this.user.uid).child("" + file.name);
    };
    Data.prototype.delete = function (data) {
        var _this = this;
        return storage().ref(data.fullPath).delete().then(function (re) {
            return _this.success(true);
        })
            .catch(function (e) { return _this.failure(e); });
    };
    return Data;
}(Base));
var Comment = /** @class */ (function (_super) {
    __extends(Comment, _super);
    function Comment() {
        var _this = _super.call(this, COLLECTIONS.COMMENTS) || this;
        _this.comments = {};
        _this.commentIds = {};
        _this._unsubscribeLikes = {};
        _this._unsubscribeComments = {};
        _this.user = new User();
        return _this;
    }
    Comment.prototype.getParentComment = function (commentId) {
        if (commentId) {
            var comment = this.getComment(commentId);
            if (comment.parentId) {
                return this.getComment(comment.parentId);
            }
            else {
                return ([]);
            }
        }
        return ([]);
    };
    Comment.prototype.getComment = function (commentId) {
        if (this.comments[commentId] === void 0) {
            return ([]);
        }
        return this.comments[commentId];
    };
    Comment.prototype.commentCollection = function (postId) {
        return this.collectionRef(COLLECTIONS.POSTS).doc(postId).collection(COLLECTIONS.COMMENTS);
    };
    Comment.prototype.comment = function (postId, commentId) {
        return this.commentCollection(postId).doc(commentId);
    };
    Comment.prototype.load = function (postId) {
        var _this = this;
        var ref = this.commentCollection(postId);
        return ref.orderBy('created', 'asc').get().then(function (s) {
            s.forEach(function (doc) {
                var c = doc.data();
                c.id = doc.id;
                _this.comments[c.id] = c;
                var sorted = _this.sortComments(postId, c);
                _this.subscribeCommentChange(postId, c);
                _this.subscribeLikes(c);
            });
            _this.subscribeCommentAdd(postId);
            return _this.commentIds[postId];
        }).catch(function (e) { return _this.failure(e); });
    };
    Comment.prototype.sortComments = function (postId, comment) {
        console.log("sortComment for: ", postId, comment);
        if (this.commentIds[postId] === void 0) {
            this.commentIds[postId] = [];
        }
        var pos = this.commentIds[postId].findIndex(function (id) { return id === comment.parentId; });
        console.log('pos: ', pos);
        if (pos === -1) {
            this.commentIds[postId].push(comment.id);
        }
        else {
            this.commentIds[postId].splice(pos + 1, 0, comment.id);
        }
        return this.commentIds[postId];
    };
    Comment.prototype.create = function (comment) {
        var _this = this;
        Library.sanitize(comment);
        comment.uid = this.user.uid;
        comment.displayName = this.user.displayName;
        comment.created = firestore.FieldValue.serverTimestamp();
        if (comment.parentId) {
            comment.depth = this.getComment(comment.parentId).depth + 1;
        }
        else {
            comment.depth = 0;
        }
        var ref = this.commentCollection(comment.postId);
        console.log("Going to add a comment under: " + ref.path);
        return ref.add(comment)
            .then(function (doc) {
            return _this.success({ id: doc.id });
        })
            .catch(function (e) {
            console.log("failed: ", e);
            _this.failure(e);
        });
    };
    Comment.prototype.edit = function (comment) {
        var _this = this;
        Library.sanitize(comment);
        comment.uid = this.user.uid;
        comment.postId = this.comments[comment.id].postId;
        comment.updated = firestore.FieldValue.serverTimestamp();
        var ref = this.comment(comment.postId, comment.id);
        console.log("Going to edit : " + ref.path + " with ", comment);
        return ref.update(comment).then(function () {
            return _this.success({ id: comment.id });
        })
            .catch(function (e) { return _this.failure(e); });
    };
    Comment.prototype.likeColllection = function (commentId, collectionName) {
        var postId = this.comments[commentId].postId;
        return this.comment(postId, commentId)
            .collection(collectionName);
    };
    Comment.prototype.like = function (commentId) {
        return this.doLike(this.likeColllection(commentId, COLLECTIONS.LIKES));
    };
    Comment.prototype.dislike = function (commentId) {
        return this.doLike(this.likeColllection(commentId, COLLECTIONS.DISLIKES));
    };
    Comment.prototype.subscribeCommentChange = function (postId, comment) {
        if (!this.settings.listenOnCommentChange) {
            return;
        }
        var unsubscribe = this.commentCollection(postId).doc(comment.id).onSnapshot(function (doc) {
            comment = Object.assign(comment, doc.data());
        });
        this.pushCommentChangeSubscriber(postId, unsubscribe);
    };
    Comment.prototype.subscribeCommentAdd = function (postId) {
        var _this = this;
        if (!this.settings.listenOnCommentChange) {
            return;
        }
        var path = this.commentCollection(postId).path;
        console.log("watch for new comment: " + path);
        var unsubscribe = this.commentCollection(postId).orderBy('created', 'desc').limit(1).onSnapshot(function (snapshot) {
            console.log('watch the lastest comment: ');
            snapshot.docChanges.forEach(function (change) {
                var doc = change.doc;
                if (doc.metadata.hasPendingWrites) {
                    console.log('pending', doc.metadata.hasPendingWrites, 'from cache: ', doc.metadata.fromCache);
                }
                else {
                    console.log('pending', doc.metadata.hasPendingWrites, 'type: ', change.type, 'from cache: ', doc.metadata.fromCache, doc.data());
                    var comment = doc.data();
                    comment.id = doc.id;
                    if (change.type === 'added' && _this.comments[comment.id] === void 0) {
                        _this.insertComment(postId, comment);
                    }
                    else if (change.type === 'modified') {
                        _this.updateComment(postId, comment);
                    }
                    else if (change.type === 'removed') {
                        _this.removeComment(comment);
                    }
                }
            });
        });
        this.pushCommentChangeSubscriber(postId, unsubscribe);
    };
    Comment.prototype.pushCommentChangeSubscriber = function (postId, unsubscribe) {
        if (this._unsubscribeComments[postId] === void 0) {
            this._unsubscribeComments[postId] = [];
        }
        this._unsubscribeComments[postId].push(unsubscribe);
    };
    Comment.prototype.insertComment = function (postId, comment) {
        if (this.comments[comment.id] === void 0) {
            console.log("insertComment: ", comment);
            this.comments[comment.id] = comment;
            this.sortComments(postId, comment);
            this.subscribeCommentChange(postId, comment);
            this.subscribeLikes(comment);
        }
    };
    Comment.prototype.updateComment = function (postId, comment) {
        console.log('updateComment id: ', comment.id);
        if (this.comments[comment.id]) {
            console.log("updateComment", comment);
            this.comments[comment.id] = Object.assign(this.comments[comment.id], comment);
        }
        else {
            this.insertComment(postId, comment);
        }
    };
    Comment.prototype.removeComment = function (comment) {
    };
    Comment.prototype.subscribeLikes = function (comment) {
        if (!this.settings.listenOnCommentLikes) {
            return;
        }
        var likeRef = this.likeColllection(comment.id, COLLECTIONS.LIKES).doc('count');
        var subscribeLike = likeRef.onSnapshot(function (doc) {
            if (doc.exists) {
                var data = doc.data();
                comment.numberOfLikes = data["count"];
            }
        });
        this.pushCommentLikeSubscriber(comment.postId, subscribeLike);
        var dislikeRef = this.likeColllection(comment.id, COLLECTIONS.DISLIKES).doc('count');
        var subscribeDislike = dislikeRef.onSnapshot(function (doc) {
            if (doc.exists) {
                var data = doc.data();
                comment.numberOfDislikes = data["count"];
            }
        });
        this.pushCommentLikeSubscriber(comment.postId, subscribeDislike);
    };
    Comment.prototype.pushCommentLikeSubscriber = function (postId, unsubscribe) {
        if (this._unsubscribeLikes[postId] === void 0) {
            this._unsubscribeLikes[postId] = [];
        }
        this._unsubscribeLikes[postId].push(unsubscribe);
    };
    Comment.prototype.unsubscribes = function (postId) {
        if (this._unsubscribeComments[postId] !== void 0 && this._unsubscribeComments[postId].length) {
            this._unsubscribeComments[postId].map(function (unsubscribe) { return unsubscribe(); });
        }
        if (this._unsubscribeLikes[postId] !== void 0 && this._unsubscribeLikes[postId].length) {
            this._unsubscribeLikes[postId].map(function (unsubscribe) { return unsubscribe(); });
        }
    };
    Comment.prototype.destory = function (post) {
        console.log("Going to destroy comments for post.id: ", post.id);
        this.unsubscribes(post.id);
    };
    return Comment;
}(Base));
var FireService = /** @class */ (function (_super) {
    __extends(FireService, _super);
    function FireService(ngZone, http) {
        var _this = _super.call(this) || this;
        _this.ngZone = ngZone;
        _this.user = new User();
        _this.category = new Category();
        _this.post = new Post();
        _this.comment = new Comment();
        _this.data = new Data();
        Base.http = http;
        _this.initUser();
        return _this;
    }
    FireService.prototype.initUser = function () {
        var _this = this;
        auth().onAuthStateChanged(function (user) {
            _this.ngZone.run(function (x) { });
        });
    };
    return FireService;
}(Base));
FireService.decorators = [
    { type: Injectable },
];
FireService.ctorParameters = function () { return [
    { type: NgZone, },
    { type: HttpClient, },
]; };
var FirelibraryModule = /** @class */ (function () {
    function FirelibraryModule() {
    }
    FirelibraryModule.forRoot = function (config) {
        Base.configure(config);
        return {
            ngModule: FirelibraryModule,
            providers: [FireService],
        };
    };
    return FirelibraryModule;
}());
FirelibraryModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    HttpClientModule
                ],
                declarations: [],
                providers: [FireService]
            },] },
];
FirelibraryModule.ctorParameters = function () { return []; };

export { FirelibraryModule, FireService, Library as _, Base, COLLECTIONS, POST_DELETED, UNKNOWN, FIREBASE_API_ERROR, NOT_FOUND, ALREADY_EXISTS, PERMISSION_DENIED, RESOURCE_EXHAUSTED, USER_IS_NOT_LOGGED_IN, NO_DOCUMENT_ID, DOCUMENT_ID_TOO_LONG, DOCUMENT_ID_CANNOT_CONTAIN_SLASH, COLLECTION_NOT_EMPTY, CATEGORY_ID_EMPTY, CATEGORY_EXISTS, CATEGORY_DOES_NOT_EXIST, POST_ID_NOT_EMPTY, POST_ID_EMPTY, ALREADY_LIKED, EMAIL_ALREADY_IN_USE, USER_NOT_FOUND, INVALID_EMAIL, WEAK_PASSWORD, EXPIRED_ID_TOKEN, WRONG_PASSWORD, PASSWORD_TOO_LONG };
//# sourceMappingURL=firelibrary.js.map
