import { Base, POST, COMMENT, COMMENT_CREATE, COMMENT_EDIT } from './../etc/base';
export declare class Comment extends Base {
    private user;
    comments: {
        [commentId: string]: COMMENT;
    };
    commentIds: {
        [postId: string]: Array<string>;
    };
    /**
     * Subscribes and unsubscribes comment's likes/dislike/changes by post.
     * This is because in some cases, only few posts may be destroyed from page and
     * it detaches the listeners of those posts only.
     */
    private _unsubscribeLikes;
    private _unsubscribeComments;
    constructor();
    /**
     * Returns parent comment.
     */
    getParentComment(commentId: any): COMMENT;
    /**
     * Returns the comment.
     */
    getComment(commentId: any): COMMENT;
    /**
     * Comment Collection under a post.
     * @param postId the post document id.
     */
    private commentCollection(postId);
    /**
     * Returns comment reference.
     */
    private comment(postId, commentId);
    /**
     * Loads all the comments under the postId.
     */
    load(postId: string): Promise<Array<string>>;
    /**
     * Sorts the comments.
     * @desc The app may need to rerender the page depending on the app context.
     * @param postId Post Document ID
     */
    sortComments(postId: string, comment: COMMENT): string[];
    /**
     * @param comment
     *      comment[postId] and comment[parentId] is already set.
     */
    create(comment: COMMENT): Promise<COMMENT_CREATE>;
    edit(comment: COMMENT): Promise<COMMENT_EDIT>;
    /**
     * Returns collection of comment like/dislike.
     * @param commentId Comment Document ID
     * @param collectionName Subcollection name under comment
     */
    private likeColllection(commentId, collectionName);
    like(commentId: string): Promise<any>;
    dislike(commentId: string): Promise<any>;
    /**
     *
     */
    private subscribeCommentChange(postId, comment);
    private subscribeCommentAdd(postId);
    private pushCommentChangeSubscriber(postId, unsubscribe);
    /**
     * Add a newly created post on top of post list on the page
     *  - and subscribe post changes if `settings.listenPostChange` is set to true.
     *  - and subscribe like/dislike based on the settings.
     *
     * @desc It's important to understand how `added` event fired on `onSnapshot)`.
     *
     */
    private insertComment(postId, comment);
    /**
     * When listening the last post on collection in realtime, it often fires `modified` event on new docuemnt created.
     */
    private updateComment(postId, comment);
    /**
     * This method is no longer in use.
     *
     * @deprecated @see README### No post delete.
     */
    private removeComment(comment);
    private subscribeLikes(comment);
    private pushCommentLikeSubscriber(postId, unsubscribe);
    /**
     * Unsubscribe for the realtime update of changes/likes/dislikes.
     * If you don't unsubscribe, you have to pay more since it will still listen(read) the documents that are no longer needed.
     */
    unsubscribes(postId: any): void;
    /**
     * Destroys all the resources that were used to display comments of a post.
     * @desc it should be called from `OnDestroy` of the comment component
     *      or anywhere you want to destroy the comments that belong to the post.
     */
    destory(post: POST): void;
}
