export interface FIRESERVICE_SETTINGS {
    /**
     * When the post has `added`, `edited`, `deleted`, the change will be updated in realtime.
     * It does not listen the change by default.
     */
    listenOnPostChange?: boolean;
    /**
     * When comments are `added`, `edited`, `deleted`, the change will be updated in realtime.
     * It does not listen the change by default.
     */
    listenOnCommentChange?: boolean;
    listenOnPostLikes?: boolean;
    listenOnCommentLikes?: boolean;
}
export interface USER {
    uid?: string;
    email: string;
    password?: string;
    displayName?: string;
    photoURL?: string;
    mobile?: string;
    city?: string;
    name?: string;
    firstName?: string;
    middleName?: string;
    lastName?: string;
    birthday?: string;
    gender?: string;
    phoneNumber?: string;
    landline?: string;
    address?: string;
    zipcode?: string;
    country?: string;
    province?: string;
    role?: string;
    created?: any;
    updated?: any;
}
/**
 * 'categories' collection schema.
 * Feel free to extends properties.
 */
export interface CATEGORY {
    route?: string;
    id: string;
    name?: string;
    description?: string;
    subcategories?: string;
    tags?: string;
    header?: string;
    footer?: string;
    numberOfPostsPerPage?: number;
    numberOfPagesOnNavigation?: number;
    moderators?: string;
    moderatorRoles?: string;
    allowAttachment?: boolean;
    levelOnList?: number;
    levelOnRead?: number;
    levelOnWrite?: number;
    disableDeleteWithDependant?: boolean;
    headerOnList?: string;
    footerOnList?: string;
    headerOnWrite?: string;
    footerOnWriter?: string;
    headerOnView?: string;
    footerOnView?: string;
    numberOfPosts?: number;
    numberOfComment?: number;
    created?: any;
    updated?: any;
}
export interface SYSTEM_CONFIG {
    firebaseApp: firebase.app.App;
    functions: boolean;
}
export interface RESPONSE {
    code: string;
    message?: string;
    data?: any;
}
export interface USER_CREATE extends RESPONSE {
    data: {
        id: string;
    };
}
export interface CATEGORY_CREATE extends RESPONSE {
    data: {
        id: string;
    };
}
export interface CATEGORY_EDIT extends RESPONSE {
    data: {
        id: string;
    };
}
export interface CATEGORY_GET extends RESPONSE {
    data: CATEGORY;
}
export interface POST_CREATE extends RESPONSE {
    data: {
        id: string;
        post: POST;
    };
}
export interface POST_EDIT extends RESPONSE {
    data: {
        id: string;
        post: POST;
    };
}
export interface POST_DELETE extends RESPONSE {
    data: {
        id: string;
    };
}
export interface COMMENT_CREATE extends RESPONSE {
    data: {
        id: string;
    };
}
export interface COMMENT_EDIT extends RESPONSE {
    data: {
        id: string;
    };
}
export interface INSTALLED extends RESPONSE {
    data: {
        installed: boolean;
    };
}
export interface USER_DATA extends RESPONSE {
    data: {
        user: USER;
    };
}
export interface POST_PAGE_OPTIONS {
    category?: string;
    limit: number;
}
export interface DATA_UPLOAD {
    name?: string;
    url?: string;
    fullPath?: string;
}
/**
* POST data to create/update/delete.
*
* @desc You can update more.
*
*
*/
export interface POST {
    id?: string;
    uid?: string;
    title?: string;
    content?: string;
    category?: string;
    subcategory?: string;
    tags?: string;
    displayName?: string;
    email?: string;
    password?: string;
    phoneNumber?: string;
    country?: string;
    province?: string;
    city?: string;
    address?: string;
    zipCode?: string;
    files?: Array<string>;
    numberOfComments?: number;
    numberOfLikes?: number;
    numberOfDislikes?: number;
    numberOfViews?: number;
    private?: boolean;
    reminder?: number;
    created?: any;
    updated?: any;
    date?: any;
    data?: Array<DATA_UPLOAD>;
    deleted?: boolean;
}
export interface COMMENT {
    id?: string;
    postId?: string;
    parentId?: string;
    uid?: string;
    displayName?: string;
    content?: string;
    created?: any;
    updated?: any;
    numberOfLikes?: number;
    numberOfDislikes?: number;
    depth?: number;
    date?: string;
    data?: Array<DATA_UPLOAD>;
    deleted?: boolean;
}
