/**
 *
 * @file library.ts
 *
 */
export declare class Library {
    /**
     * Returns http query string.
     * @param params Object to build as http query string
     * @return
     *      - http query string
     *      - Or null if the input is emtpy or not object.
     */
    static httpBuildQuery(params: any): string | null;
    /**
     * Returns n'th portion of the input `str` after spliting by the `separator`
     *
     * @param str string to get a portion from.
     * @param separator to split the string. Default is a Blank.
     * @param n n'th portion to get. Index begins with 0. Default is 0.
     * @return
     *      - a portion of the input string.
     *      - or null
     *          - if the input `str` is empty.
     *          - if the input `str` is not a string.
     *          - if the n'th portion does not exists.
     *          - if the value of the portion is empty
     *          - if separator is not a string and empty.
     *
     * @code
     *      const str = 'abc.def.ghi';
     *      return this.library.segment( str, '.', 0 ); // returns `abc`
     *
     */
    static segment(str: string, separator?: string, n?: number): string;
    /**
     * Returns true if the input `what` is falsy or empty or no data.
     * @returns true if the input `what` is
     *          - falsy value.
     *              -- boolean and it's false,
     *              -- number with 0.
     *              -- string with empty. ( if it has any vlaue like blank, then it's not empty. )
     *              -- undefined.
     *          - object with no key.
     *          - array with 0 length.
     *
     *      - otherwise return false.
     */
    static isEmpty(what: any): boolean;
    /**
     * Returns true if the input `a` and `b` are identical.
     * @param a Object a
     * @param b Object b
     */
    static isEqual(a: any, b: any): boolean;
    static isString(str: any): boolean;
    /**
     *
     * Removes properties with `undefined` value from the object and returns it.
     *
     * You cannot set `undefiend` value into firestore `document`. It will produce a Critical error.
     *
     * @param obj Object to be set into `firestore`.
     *      It is passed by reference.
     *
     * @return the input object that has sanitized.
     */
    static sanitize(obj: any): any;
    /**
     * Removes space(s) between the separator in `separator`
     * @description
     *      If the input str is given with `a, b, c c ,d `, then the return will be `a,b,c c,d`.
     * @param separator separator in string
     * @param str string to remove space from.
     *
     * @returns a string after removing spaces between the `separator`.
     *      - if the string is falsy, it returns the input `str` itself.
     */
    static removeSpaceBetween(separator: string, str: string): string;
    /**
     * Returns browser language
     *
     * @param full If it is true, then it returns the full language string like 'en-US'.
     *              Otherwise, it returns the first two letters like 'en'.
     *
     * @returns
     *      - the browser language like 'en', 'en-US', 'ko', 'ko-KR'
     *      - null if it cannot detect a language.
     */
    static getBrowserLanguage(full?: boolean): string;
    /**
     *
     * Returns a string after patching error information.
     * @param str Error string
     * @param info Error information to patch into the string
     *
     *
     *
     * @return patched string
     *
     * @code
     *      _.patchmarker( 'Unknown #no', {no: 123} ) // returns 'Unknown 123'
     *
     */
    static patchMarker(str: any, info?: object): string;
}
