import { Base, USER, RESPONSE, USER_CREATE, USER_DATA } from './../etc/base';
import * as firebase from 'firebase';
export declare class User extends Base {
    constructor();
    readonly isLogin: boolean;
    readonly isLogout: boolean;
    /**Get current user's uid. */
    readonly uid: string;
    readonly email: string;
    readonly displayName: string;
    readonly photoURL: string;
    /**
     * Returns user data
     */
    data(): Promise<USER_DATA>;
    /**
    * @desc Validates user data to be used for registration
    * @param user User data to validate
    */
    registerValidator(user: USER): Promise<any>;
    /**
    * User regigration with email and password.
    * @desc this method get user data from HTML FORM including email, password, displayName, gender, birthday, etc.
    * @desc `Firebase Authentication` needs to create an `Authentication` in their `Authentication Service`.
    *      The app
    *          1. Create an Authentication (on the Authentication Service)
    *          2. Update the profile on the Authentication with displayName and photoURL.
    *          3. Sets other information on `users` collection.
    */
    register(data: USER): Promise<RESPONSE>;
    /**
    * Validator for User.login()
    */
    loginValidator(email: string, password: string): Promise<any>;
    /**
    * Login User
    * @param email
    * @param password
    */
    login(email: string, password: string): Promise<any>;
    /**
    * Logout user.
    */
    logout(): void;
    /**
    * Update `displayName`, `photoURL` on Authentication.
    * @desc it does not update other information.
    * @see `this.updateProfile()` for updating user information.
    */
    updateAuthentication(user: firebase.User, data: USER): Promise<firebase.User>;
    /**
    * Sets user information on user collection.
    */
    set(user: firebase.User, data: USER): Promise<USER_CREATE>;
    create(data: USER): Promise<USER_CREATE>;
    update(data: USER): Promise<USER_CREATE>;
    delete(): Promise<any>;
}
